@@ -23,13 +23,15 @@ import (
 	"net"
 	"os"
 	"strconv"
+	"strings"
 	"time"
 
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/credentials"
+	"google.golang.org/grpc/credentials/insecure"
 
-	api "github.com/osrg/gobgp/api"
-	"github.com/osrg/gobgp/pkg/packet/bgp"
+	api "github.com/osrg/gobgp/v3/api"
+	"github.com/osrg/gobgp/v3/pkg/packet/bgp"
 )
 
 const globalRIBName = "global"
@@ -204,14 +206,21 @@ func newClient(ctx context.Context) (api.GobgpApiClient, context.CancelFunc, err
 		}
 		grpcOpts = append(grpcOpts, grpc.WithTransportCredentials(creds))
 	} else {
-		grpcOpts = append(grpcOpts, grpc.WithInsecure())
+		grpcOpts = append(grpcOpts, grpc.WithTransportCredentials(insecure.NewCredentials()))
 	}
 
-	target := net.JoinHostPort(globalOpts.Host, strconv.Itoa(globalOpts.Port))
+	target := globalOpts.Target
 	if target == "" {
-		target = ":50051"
+		target = net.JoinHostPort(globalOpts.Host, strconv.Itoa(globalOpts.Port))
+	} else if strings.HasPrefix(target, "unix://") {
+		target = target[len("unix://"):]
+		dialer := func(ctx context.Context, addr string) (net.Conn, error) {
+			return net.Dial("unix", addr)
+		}
+		grpcOpts = append(grpcOpts, grpc.WithContextDialer(dialer))
 	}
 	cc, cancel := context.WithTimeout(ctx, time.Second)
+
 	conn, err := grpc.DialContext(cc, target, grpcOpts...)
 	if err != nil {
 		return nil, cancel, err
@@ -303,6 +312,14 @@ var (
 		Afi:  api.Family_AFI_LS,
 		Safi: api.Family_SAFI_LS,
 	}
+	ipv4MUP = &api.Family{
+		Afi:  api.Family_AFI_IP,
+		Safi: api.Family_SAFI_MUP,
+	}
+	ipv6MUP = &api.Family{
+		Afi:  api.Family_AFI_IP6,
+		Safi: api.Family_SAFI_MUP,
+	}
 )
 
 func checkAddressFamily(def *api.Family) (*api.Family, error) {
@@ -343,6 +360,10 @@ func checkAddressFamily(def *api.Family) (*api.Family, error) {
 		f = opaque
 	case "ls", "linkstate", "bgpls":
 		f = ls
+	case "ipv4-mup", "mup-ipv4", "mup4":
+		f = ipv4MUP
+	case "ipv6-mup", "mup-ipv6", "mup6":
+		f = ipv6MUP
 	case "":
 		f = def
 	default:
