@@ -24,10 +24,9 @@ import (
 	"sort"
 	"time"
 
-	"github.com/osrg/gobgp/internal/pkg/config"
-	"github.com/osrg/gobgp/pkg/packet/bgp"
-
-	log "github.com/sirupsen/logrus"
+	"github.com/osrg/gobgp/v3/internal/pkg/config"
+	"github.com/osrg/gobgp/v3/pkg/log"
+	"github.com/osrg/gobgp/v3/pkg/packet/bgp"
 )
 
 const (
@@ -71,9 +70,7 @@ func (b *Bitmap) FindandSetZeroBit() (uint, error) {
 func (b *Bitmap) Expand() {
 	old := b.bitmap
 	new := make([]uint64, len(old)+1)
-	for i := 0; i < len(old); i++ {
-		new[i] = old[i]
-	}
+	copy(new, old)
 	b.bitmap = new
 }
 
@@ -153,10 +150,6 @@ func NewPath(source *PeerInfo, nlri bgp.AddrPrefixInterface, isWithdraw bool, pa
 		source = localSource
 	}
 	if !isWithdraw && pattrs == nil {
-		log.WithFields(log.Fields{
-			"Topic": "Table",
-			"Key":   nlri.String(),
-		}).Error("Need to provide path attributes for non-withdrawn path.")
 		return nil
 	}
 
@@ -201,7 +194,7 @@ func cloneAsPath(asAttr *bgp.PathAttributeAsPath) *bgp.PathAttributeAsPath {
 	return bgp.NewPathAttributeAsPath(newASparams)
 }
 
-func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInfo, original *Path) *Path {
+func UpdatePathAttrs(logger log.Logger, global *config.Global, peer *config.Neighbor, info *PeerInfo, original *Path) *Path {
 	if peer.RouteServer.Config.RouteServerClient {
 		return original
 	}
@@ -307,10 +300,11 @@ func UpdatePathAttrs(global *config.Global, peer *config.Neighbor, info *PeerInf
 		}
 
 	} else {
-		log.WithFields(log.Fields{
-			"Topic": "Peer",
-			"Key":   peer.State.NeighborAddress,
-		}).Warnf("invalid peer type: %v", peer.State.PeerType)
+		logger.Warn("invalid peer type",
+			log.Fields{
+				"Topic": "Peer",
+				"Key":   peer.State.NeighborAddress,
+				"Type":  peer.State.PeerType})
 	}
 	return path
 }
@@ -653,25 +647,26 @@ func (path *Path) GetLabelString() string {
 // PrependAsn prepends AS number.
 // This function updates the AS_PATH attribute as follows.
 // (If the peer is in the confederation member AS,
-//  replace AS_SEQUENCE in the following sentence with AS_CONFED_SEQUENCE.)
-//  1) if the first path segment of the AS_PATH is of type
-//     AS_SEQUENCE, the local system prepends the specified AS num as
-//     the last element of the sequence (put it in the left-most
-//     position with respect to the position of  octets in the
-//     protocol message) the specified number of times.
-//     If the act of prepending will cause an overflow in the AS_PATH
-//     segment (i.e.,  more than 255 ASes),
-//     it SHOULD prepend a new segment of type AS_SEQUENCE
-//     and prepend its own AS number to this new segment.
 //
-//  2) if the first path segment of the AS_PATH is of other than type
-//     AS_SEQUENCE, the local system prepends a new path segment of type
-//     AS_SEQUENCE to the AS_PATH, including the specified AS number in
-//     that segment.
+//	replace AS_SEQUENCE in the following sentence with AS_CONFED_SEQUENCE.)
+//	1) if the first path segment of the AS_PATH is of type
+//	   AS_SEQUENCE, the local system prepends the specified AS num as
+//	   the last element of the sequence (put it in the left-most
+//	   position with respect to the position of  octets in the
+//	   protocol message) the specified number of times.
+//	   If the act of prepending will cause an overflow in the AS_PATH
+//	   segment (i.e.,  more than 255 ASes),
+//	   it SHOULD prepend a new segment of type AS_SEQUENCE
+//	   and prepend its own AS number to this new segment.
 //
-//  3) if the AS_PATH is empty, the local system creates a path
-//     segment of type AS_SEQUENCE, places the specified AS number
-//     into that segment, and places that segment into the AS_PATH.
+//	2) if the first path segment of the AS_PATH is of other than type
+//	   AS_SEQUENCE, the local system prepends a new path segment of type
+//	   AS_SEQUENCE to the AS_PATH, including the specified AS number in
+//	   that segment.
+//
+//	3) if the AS_PATH is empty, the local system creates a path
+//	   segment of type AS_SEQUENCE, places the specified AS number
+//	   into that segment, and places that segment into the AS_PATH.
 func (path *Path) PrependAsn(asn uint32, repeat uint8, confed bool) {
 	var segType uint8
 	if confed {
@@ -1100,6 +1095,18 @@ func (v *Vrf) ToGlobalPath(path *Path) error {
 		case bgp.EVPN_INCLUSIVE_MULTICAST_ETHERNET_TAG:
 			n.RouteTypeData.(*bgp.EVPNMulticastEthernetTagRoute).RD = v.Rd
 		}
+	case bgp.RF_MUP_IPv4, bgp.RF_MUP_IPv6:
+		n := nlri.(*bgp.MUPNLRI)
+		switch n.RouteType {
+		case bgp.MUP_ROUTE_TYPE_INTERWORK_SEGMENT_DISCOVERY:
+			n.RouteTypeData.(*bgp.MUPInterworkSegmentDiscoveryRoute).RD = v.Rd
+		case bgp.MUP_ROUTE_TYPE_DIRECT_SEGMENT_DISCOVERY:
+			n.RouteTypeData.(*bgp.MUPDirectSegmentDiscoveryRoute).RD = v.Rd
+		case bgp.MUP_ROUTE_TYPE_TYPE_1_SESSION_TRANSFORMED:
+			n.RouteTypeData.(*bgp.MUPType1SessionTransformedRoute).RD = v.Rd
+		case bgp.MUP_ROUTE_TYPE_TYPE_2_SESSION_TRANSFORMED:
+			n.RouteTypeData.(*bgp.MUPType2SessionTransformedRoute).RD = v.Rd
+		}
 	default:
 		return fmt.Errorf("unsupported route family for vrf: %s", rf)
 	}
@@ -1146,6 +1153,22 @@ func (p *Path) ToGlobal(vrf *Vrf) *Path {
 			}
 			nlri = bgp.NewEVPNNLRI(n.RouteType, new)
 		}
+	case bgp.RF_MUP_IPv4, bgp.RF_MUP_IPv6:
+		n := nlri.(*bgp.MUPNLRI)
+		switch n.RouteType {
+		case bgp.MUP_ROUTE_TYPE_INTERWORK_SEGMENT_DISCOVERY:
+			old := n.RouteTypeData.(*bgp.MUPInterworkSegmentDiscoveryRoute)
+			nlri = bgp.NewMUPInterworkSegmentDiscoveryRoute(vrf.Rd, old.Prefix)
+		case bgp.MUP_ROUTE_TYPE_DIRECT_SEGMENT_DISCOVERY:
+			old := n.RouteTypeData.(*bgp.MUPDirectSegmentDiscoveryRoute)
+			nlri = bgp.NewMUPDirectSegmentDiscoveryRoute(vrf.Rd, old.Address)
+		case bgp.MUP_ROUTE_TYPE_TYPE_1_SESSION_TRANSFORMED:
+			old := n.RouteTypeData.(*bgp.MUPType1SessionTransformedRoute)
+			nlri = bgp.NewMUPType1SessionTransformedRoute(vrf.Rd, old.Prefix, old.TEID, old.QFI, old.EndpointAddress)
+		case bgp.MUP_ROUTE_TYPE_TYPE_2_SESSION_TRANSFORMED:
+			old := n.RouteTypeData.(*bgp.MUPType2SessionTransformedRoute)
+			nlri = bgp.NewMUPType2SessionTransformedRoute(vrf.Rd, old.EndpointAddress, old.TEID)
+		}
 	default:
 		return p
 	}
@@ -1159,28 +1182,33 @@ func (p *Path) ToGlobal(vrf *Vrf) *Path {
 func (p *Path) ToLocal() *Path {
 	nlri := p.GetNlri()
 	f := p.GetRouteFamily()
-	pathId := nlri.PathLocalIdentifier()
+	localPathId := nlri.PathLocalIdentifier()
+	pathId := nlri.PathIdentifier()
 	switch f {
 	case bgp.RF_IPv4_VPN:
 		n := nlri.(*bgp.LabeledVPNIPAddrPrefix)
 		_, c, _ := net.ParseCIDR(n.IPPrefix())
 		ones, _ := c.Mask.Size()
 		nlri = bgp.NewIPAddrPrefix(uint8(ones), c.IP.String())
-		nlri.SetPathLocalIdentifier(pathId)
+		nlri.SetPathLocalIdentifier(localPathId)
+		nlri.SetPathIdentifier(pathId)
 	case bgp.RF_FS_IPv4_VPN:
 		n := nlri.(*bgp.FlowSpecIPv4VPN)
 		nlri = bgp.NewFlowSpecIPv4Unicast(n.FlowSpecNLRI.Value)
-		nlri.SetPathLocalIdentifier(pathId)
+		nlri.SetPathLocalIdentifier(localPathId)
+		nlri.SetPathIdentifier(pathId)
 	case bgp.RF_IPv6_VPN:
 		n := nlri.(*bgp.LabeledVPNIPv6AddrPrefix)
 		_, c, _ := net.ParseCIDR(n.IPPrefix())
 		ones, _ := c.Mask.Size()
 		nlri = bgp.NewIPv6AddrPrefix(uint8(ones), c.IP.String())
-		nlri.SetPathLocalIdentifier(pathId)
+		nlri.SetPathLocalIdentifier(localPathId)
+		nlri.SetPathIdentifier(pathId)
 	case bgp.RF_FS_IPv6_VPN:
 		n := nlri.(*bgp.FlowSpecIPv6VPN)
 		nlri = bgp.NewFlowSpecIPv6Unicast(n.FlowSpecNLRI.Value)
-		nlri.SetPathLocalIdentifier(pathId)
+		nlri.SetPathLocalIdentifier(localPathId)
+		nlri.SetPathIdentifier(pathId)
 	default:
 		return p
 	}
@@ -1233,12 +1261,22 @@ func nlriToIPNet(nlri bgp.AddrPrefixInterface) *net.IPNet {
 	case *bgp.LabeledIPAddrPrefix:
 		return &net.IPNet{
 			IP:   net.IP(T.Prefix.To4()),
-			Mask: net.CIDRMask(int(T.Length), 32),
+			Mask: net.CIDRMask(int(T.Length)-T.Labels.Len()*8, 32),
 		}
 	case *bgp.LabeledIPv6AddrPrefix:
+		return &net.IPNet{
+			IP:   net.IP(T.Prefix.To16()),
+			Mask: net.CIDRMask(int(T.Length)-T.Labels.Len()*8, 128),
+		}
+	case *bgp.LabeledVPNIPAddrPrefix:
 		return &net.IPNet{
 			IP:   net.IP(T.Prefix.To4()),
-			Mask: net.CIDRMask(int(T.Length), 128),
+			Mask: net.CIDRMask(int(T.Length)-T.Labels.Len()*8-T.RD.Len()*8, 32),
+		}
+	case *bgp.LabeledVPNIPv6AddrPrefix:
+		return &net.IPNet{
+			IP:   net.IP(T.Prefix.To16()),
+			Mask: net.CIDRMask(int(T.Length)-T.Labels.Len()*8-T.RD.Len()*8, 128),
 		}
 	}
 	return nil
