@@ -6,57 +6,61 @@ import (
 	"encoding/json"
 	"fmt"
 	"net"
-
-	"github.com/golang/protobuf/ptypes"
-	api "github.com/osrg/gobgp/api"
+	"net/netip"
 )
 
 const (
 	prefixSIDtlvHdrLen = 4
 )
 
+// BGP Prefix-SID TLV Types
+// https://www.iana.org/assignments/bgp-parameters/bgp-parameters.xhtml#bgp-prefix-sid-tlv-types
 type TLVType uint8
 
+const (
+	_ TLVType = iota // Reserved
+	TLVTypeLavelIndex
+	_ // Deprecated
+	TLVTypeOriginatorSRGB
+	_ // Deprecated
+	TLVTypeSRv6L3Service
+	TLVTypeSRv6L2Service
+)
+
 type TLV struct {
 	Type   TLVType
 	Length uint16
 }
 
-func (s *TLV) Len() int {
-	return int(s.Length) + tlvHdrLen - 1 // Extra reserved byte in the header
+func (t *TLV) Len() int {
+	return int(t.Length) + 3 // Type(1) + Length(2)
 }
 
-func (s *TLV) Serialize(value []byte) ([]byte, error) {
-	if len(value) != int(s.Length)-1 {
+func (t *TLV) Serialize(value []byte) ([]byte, error) {
+	if len(value) != int(t.Len()) {
 		return nil, malformedAttrListErr("serialization failed: Prefix SID TLV malformed")
 	}
-	buf := make([]byte, prefixSIDtlvHdrLen+len(value))
 	p := 0
-	buf[p] = byte(s.Type)
+	value[p] = byte(t.Type)
 	p++
-	binary.BigEndian.PutUint16(buf[p:p+2], uint16(s.Length))
-	p += 2
-	// Reserved byte
-	p++
-	copy(buf[p:], value)
-
-	return buf, nil
+	binary.BigEndian.PutUint16(value[p:p+2], uint16(t.Length))
+	return value, nil
 }
 
-func (s *TLV) DecodeFromBytes(data []byte) ([]byte, error) {
-	if len(data) < prefixSIDtlvHdrLen {
+func (t *TLV) DecodeFromBytes(data []byte) ([]byte, error) {
+	if len(data) < 3 {
 		return nil, malformedAttrListErr("decoding failed: Prefix SID TLV malformed")
 	}
 	p := 0
-	s.Type = TLVType(data[p])
+	t.Type = TLVType(data[p])
 	p++
-	s.Length = binary.BigEndian.Uint16(data[p : p+2])
+	t.Length = binary.BigEndian.Uint16(data[p : p+2])
+	p += 2
 
-	if s.Len() < prefixSIDtlvHdrLen || len(data) < s.Len() {
+	if len(data[p:]) < int(t.Length) {
 		return nil, malformedAttrListErr("decoding failed: Prefix SID TLV malformed")
 	}
-
-	return data[prefixSIDtlvHdrLen:s.Len()], nil
+	return data[p : p+int(t.Length)], nil
 }
 
 // PrefixSIDTLVInterface defines standard set of methods to handle Prefix SID attribute's TLVs
@@ -77,6 +81,21 @@ type PathAttributePrefixSID struct {
 	TLVs []PrefixSIDTLVInterface
 }
 
+func NewPathAttributePrefixSID(values ...PrefixSIDTLVInterface) *PathAttributePrefixSID {
+	var l int
+	for _, v := range values {
+		l += v.Len()
+	}
+	return &PathAttributePrefixSID{
+		PathAttribute: PathAttribute{
+			Flags:  getPathAttrFlags(BGP_ATTR_TYPE_PREFIX_SID, l),
+			Type:   BGP_ATTR_TYPE_PREFIX_SID,
+			Length: uint16(l),
+		},
+		TLVs: values,
+	}
+}
+
 func (p *PathAttributePrefixSID) DecodeFromBytes(data []byte, options ...*MarshallingOption) error {
 	tlvs, err := p.PathAttribute.DecodeFromBytes(data)
 	if err != nil {
@@ -92,8 +111,8 @@ func (p *PathAttributePrefixSID) DecodeFromBytes(data []byte, options ...*Marsha
 
 		var tlv PrefixSIDTLVInterface
 		switch t.Type {
-		case 5:
-			tlv = &SRv6L3ServiceAttribute{
+		case TLVTypeSRv6L3Service, TLVTypeSRv6L2Service:
+			tlv = &SRv6ServiceTLV{
 				SubTLVs: make([]PrefixSIDTLVInterface, 0),
 			}
 		default:
@@ -161,6 +180,7 @@ type SRv6L3Service struct {
 }
 
 // SRv6L3ServiceAttribute defines the structure of SRv6 L3 Service attribute
+// Deprecated: Use SRv6ServiceTLV instead.
 type SRv6L3ServiceAttribute struct {
 	TLV
 	SubTLVs []PrefixSIDTLVInterface
@@ -171,13 +191,15 @@ func (s *SRv6L3ServiceAttribute) Len() int {
 }
 
 func (s *SRv6L3ServiceAttribute) Serialize() ([]byte, error) {
-	buf := make([]byte, 0)
+	buf := make([]byte, s.Length+3)
+	p := 4
 	for _, tlv := range s.SubTLVs {
 		s, err := tlv.Serialize()
 		if err != nil {
 			return nil, err
 		}
-		buf = append(buf, s...)
+		copy(buf[p:p+len(s)], s)
+		p += len(s)
 	}
 	return s.TLV.Serialize(buf)
 }
@@ -187,6 +209,7 @@ func (s *SRv6L3ServiceAttribute) DecodeFromBytes(data []byte) error {
 	if err != nil {
 		return err
 	}
+	stlvs = stlvs[1:] // RESERVED(1)
 
 	for len(stlvs) >= subTLVHdrLen {
 		t := &SubTLV{}
@@ -277,13 +300,13 @@ func (s *SubTLV) Serialize(value []byte) ([]byte, error) {
 
 func (s *SubTLV) DecodeFromBytes(data []byte) ([]byte, error) {
 	if len(data) < subTLVHdrLen {
-		return nil, malformedAttrListErr("decoding failed: Prefix SID TLV malformed")
+		return nil, malformedAttrListErr("decoding failed: Prefix SID Sub TLV malformed")
 	}
 	s.Type = SubTLVType(data[0])
 	s.Length = binary.BigEndian.Uint16(data[1:3])
 
 	if len(data) < s.Len() {
-		return nil, malformedAttrListErr("decoding failed: Prefix SID TLV malformed")
+		return nil, malformedAttrListErr("decoding failed: Prefix SID Sub TLV malformed")
 	}
 
 	return data[subTLVHdrLen:s.Len()], nil
@@ -306,6 +329,23 @@ type SRv6InformationSubTLV struct {
 	SubSubTLVs       []PrefixSIDTLVInterface
 }
 
+func NewSRv6InformationSubTLV(sid netip.Addr, behavior SRBehavior, values ...PrefixSIDTLVInterface) *SRv6InformationSubTLV {
+	l := 21 // RESERVED1(1) + SID(16) + Flags(1) + Endpoint Behavior(2) + RESERVED2(1)
+	for _, v := range values {
+		l += v.Len()
+	}
+	return &SRv6InformationSubTLV{
+		SubTLV: SubTLV{
+			Type:   1,
+			Length: uint16(l),
+		},
+		SID:              sid.AsSlice(),
+		Flags:            0,
+		EndpointBehavior: uint16(behavior),
+		SubSubTLVs:       values,
+	}
+}
+
 func (s *SRv6InformationSubTLV) Len() int {
 	return int(s.Length) + subTLVHdrLen
 }
@@ -463,10 +503,10 @@ func (s *SubSubTLV) DecodeFromBytes(data []byte) ([]byte, error) {
 }
 
 // SRv6SIDStructureSubSubTLV defines a structure of SRv6 SID Structure Sub Sub TLV (type 1) object
-// https://tools.ietf.org/html/draft-dawra-bess-srv6-services-02#section-2.1.2.1
+// https://www.rfc-editor.org/rfc/rfc9252.html#section-3.2.1
 type SRv6SIDStructureSubSubTLV struct {
 	SubSubTLV
-	LocalBlockLength    uint8
+	LocatorBlockLength  uint8
 	LocatorNodeLength   uint8
 	FunctionLength      uint8
 	ArgumentLength      uint8
@@ -474,6 +514,21 @@ type SRv6SIDStructureSubSubTLV struct {
 	TranspositionOffset uint8
 }
 
+func NewSRv6SIDStructureSubSubTLV(lbl, lnl, fl, al, tl, to uint8) *SRv6SIDStructureSubSubTLV {
+	return &SRv6SIDStructureSubSubTLV{
+		SubSubTLV: SubSubTLV{
+			Type:   1,
+			Length: 6,
+		},
+		LocatorBlockLength:  lbl,
+		LocatorNodeLength:   lnl,
+		FunctionLength:      fl,
+		ArgumentLength:      al,
+		TranspositionLength: tl,
+		TranspositionOffset: to,
+	}
+}
+
 func (s *SRv6SIDStructureSubSubTLV) Len() int {
 	return int(s.Length) + subSubTLVHdrLen
 }
@@ -481,7 +536,7 @@ func (s *SRv6SIDStructureSubSubTLV) Len() int {
 func (s *SRv6SIDStructureSubSubTLV) Serialize() ([]byte, error) {
 	buf := make([]byte, s.Length)
 	p := 0
-	buf[p] = s.LocalBlockLength
+	buf[p] = s.LocatorBlockLength
 	p++
 	buf[p] = s.LocatorNodeLength
 	p++
@@ -503,7 +558,7 @@ func (s *SRv6SIDStructureSubSubTLV) DecodeFromBytes(data []byte) error {
 	s.Type = SubSubTLVType(data[0])
 	s.Length = binary.BigEndian.Uint16(data[1:3])
 
-	s.LocalBlockLength = data[3]
+	s.LocatorBlockLength = data[3]
 	s.LocatorNodeLength = data[4]
 	s.FunctionLength = data[5]
 	s.ArgumentLength = data[6]
@@ -516,7 +571,7 @@ func (s *SRv6SIDStructureSubSubTLV) DecodeFromBytes(data []byte) error {
 func (s *SRv6SIDStructureSubSubTLV) MarshalJSON() ([]byte, error) {
 	return json.Marshal(struct {
 		Type                SubSubTLVType `json:"type"`
-		LocalBlockLength    uint8         `json:"local_block_length"`
+		LocatorBlockLength  uint8         `json:"locator_block_length"`
 		LocatorNodeLength   uint8         `json:"locator_node_length"`
 		FunctionLength      uint8         `json:"function_length"`
 		ArgumentLength      uint8         `json:"argument_length"`
@@ -524,7 +579,7 @@ func (s *SRv6SIDStructureSubSubTLV) MarshalJSON() ([]byte, error) {
 		TranspositionOffset uint8         `json:"transposition_offset"`
 	}{
 		Type:                s.Type,
-		LocalBlockLength:    s.LocalBlockLength,
+		LocatorBlockLength:  s.LocatorBlockLength,
 		LocatorNodeLength:   s.LocatorNodeLength,
 		FunctionLength:      s.FunctionLength,
 		ArgumentLength:      s.ArgumentLength,
@@ -534,8 +589,8 @@ func (s *SRv6SIDStructureSubSubTLV) MarshalJSON() ([]byte, error) {
 }
 
 func (s *SRv6SIDStructureSubSubTLV) String() string {
-	return fmt.Sprintf("{SRv6 Structure Sub Sub TLV: [ Local Block Length: %d, Locator Node Length: %d, Function Length: %d, Argument Length: %d, Transposition Length: %d, Transposition Offset: %d] }",
-		s.LocalBlockLength,
+	return fmt.Sprintf("{SRv6 Structure Sub Sub TLV: [ Locator Block Length: %d, Locator Node Length: %d, Function Length: %d, Argument Length: %d, Transposition Length: %d, Transposition Offset: %d] }",
+		s.LocatorBlockLength,
 		s.LocatorNodeLength,
 		s.FunctionLength,
 		s.ArgumentLength,
@@ -544,134 +599,96 @@ func (s *SRv6SIDStructureSubSubTLV) String() string {
 	)
 }
 
-func NewPathAttributePrefixSID(psid *api.PrefixSID) (*PathAttributePrefixSID, error) {
-	t := BGP_ATTR_TYPE_PREFIX_SID
-	s := &PathAttributePrefixSID{
-		PathAttribute: PathAttribute{
-			Flags: PathAttrFlags[t],
-			Type:  t,
-		},
-		TLVs: make([]PrefixSIDTLVInterface, 0),
+// SRv6ServiceTLV represents SRv6 Service TLV.
+// https://www.rfc-editor.org/rfc/rfc9252.html#section-2
+type SRv6ServiceTLV struct {
+	TLV
+	SubTLVs []PrefixSIDTLVInterface
+}
+
+func NewSRv6ServiceTLV(t TLVType, values ...PrefixSIDTLVInterface) *SRv6ServiceTLV {
+	l := 1 // RESERVED(1)
+	for _, v := range values {
+		l += v.Len()
 	}
-	for _, raw := range psid.Tlvs {
-		var tlv ptypes.DynamicAny
-		if err := ptypes.UnmarshalAny(raw, &tlv); err != nil {
+	return &SRv6ServiceTLV{
+		TLV: TLV{
+			Type:   t,
+			Length: uint16(l),
+		},
+		SubTLVs: values,
+	}
+}
+
+func (s *SRv6ServiceTLV) Len() int {
+	return int(s.Length) + 3 // Type(1) + Length(2)
+}
+
+func (t *SRv6ServiceTLV) Serialize() ([]byte, error) {
+	buf := make([]byte, t.Len())
+	p := 4
+	for _, tlv := range t.SubTLVs {
+		b, err := tlv.Serialize()
+		if err != nil {
 			return nil, err
 		}
-		switch v := tlv.Message.(type) {
-		case *api.SRv6L3ServiceTLV:
-			tlvLength, tlvs, err := UnmarshalSubTLVs(v.SubTlvs)
-			if err != nil {
-				return nil, err
-			}
-			o := &SRv6L3ServiceAttribute{
-				TLV: TLV{
-					Type:   TLVType(5),
-					Length: tlvLength,
-				},
-			}
-			s.PathAttribute.Length += tlvLength
-			// Storing Sub TLVs in a Service TLV
-			o.SubTLVs = append(o.SubTLVs, tlvs...)
-			// Adding Service TLV to Path Attribute TLV slice.
-			s.TLVs = append(s.TLVs, o)
-		default:
-			return nil, fmt.Errorf("unknown or not implemented Prefix SID type: %+v", v)
-		}
+		copy(buf[p:p+len(b)], b)
+		p += len(b)
 	}
-	// Final Path Attribute Length is 3 bytes of the header and 1 byte Reserved1
-	s.PathAttribute.Length += (3 + 1)
-	return s, nil
+	return t.TLV.Serialize(buf)
 }
 
-func UnmarshalSubTLVs(stlvs map[uint32]*api.SRv6TLV) (uint16, []PrefixSIDTLVInterface, error) {
-	p := make([]PrefixSIDTLVInterface, 0, len(stlvs))
-	l := uint16(0)
-	// v.SubTlvs is a map by sub tlv type and the value is a slice of sub tlvs of the specific type
-	for t, tlv := range stlvs {
-		switch t {
+func (s *SRv6ServiceTLV) DecodeFromBytes(data []byte) error {
+	stlvs, err := s.TLV.DecodeFromBytes(data)
+	if err != nil {
+		return err
+	}
+	stlvs = stlvs[1:] // RESERVED(1)
+
+	for len(stlvs) >= subTLVHdrLen {
+		t := &SubTLV{}
+		_, err := t.DecodeFromBytes(stlvs)
+		if err != nil {
+			return err
+		}
+
+		var stlv PrefixSIDTLVInterface
+		switch t.Type {
 		case 1:
-			// Sub TLV Type 1 is SRv6 Informational Sub TLV
-			for _, stlvRaw := range tlv.Tlv {
-				// Instantiating Information Sub TLV
-				info := &SRv6InformationSubTLV{
-					SubTLV: SubTLV{
-						Type: SubTLVType(1),
-					},
-					SubSubTLVs: make([]PrefixSIDTLVInterface, 0),
-				}
-				var raw ptypes.DynamicAny
-				if err := ptypes.UnmarshalAny(stlvRaw, &raw); err != nil {
-					return 0, nil, err
-				}
-				infoProto := raw.Message.(*api.SRv6InformationSubTLV)
-				info.SID = make([]byte, len(infoProto.Sid))
-				copy(info.SID, infoProto.Sid)
-				// TODO Once RFC is published add processing of flags
-				info.Flags = 0
-				info.EndpointBehavior = uint16(infoProto.EndpointBehavior)
-				var sstlvslength uint16
-				var sstlvs []PrefixSIDTLVInterface
-				if len(infoProto.SubSubTlvs) != 0 {
-					// Processing Sub Sub TLVs
-					var err error
-					sstlvslength, sstlvs, err = UnmarshalSubSubTLVs(infoProto.SubSubTlvs)
-					if err != nil {
-						return 0, nil, err
-					}
-					info.SubSubTLVs = append(info.SubSubTLVs, sstlvs...)
-				}
-				// SRv6 Information Sub TLV length consists 1 byte Resrved2, 16 bytes SID, 1 byte flags, 2 bytes Endpoint Behavior
-				// 1 byte Reserved3 and length of Sub Sub TLVs
-				info.SubTLV.Length = 1 + 16 + 1 + 2 + 1 + sstlvslength
-				// For total Srv6 Information Sub TLV length, adding 3 bytes of the Sub TLV header
-				l += info.SubTLV.Length + 4
-				p = append(p, info)
+			stlv = &SRv6InformationSubTLV{
+				SubSubTLVs: make([]PrefixSIDTLVInterface, 0),
 			}
 		default:
-			return 0, nil, fmt.Errorf("unknown or not implemented Prefix SID Sub TLV type: %d", t)
+			data = data[t.Len():]
+			continue
 		}
+
+		if err := stlv.DecodeFromBytes(stlvs); err != nil {
+			return err
+		}
+		stlvs = stlvs[t.Len():]
+		s.SubTLVs = append(s.SubTLVs, stlv)
 	}
 
-	return l, p, nil
+	return nil
 }
 
-func UnmarshalSubSubTLVs(stlvs map[uint32]*api.SRv6TLV) (uint16, []PrefixSIDTLVInterface, error) {
-	p := make([]PrefixSIDTLVInterface, 0)
-	l := uint16(0)
-	// v.SubTlvs is a map by sub tlv type and the value is a slice of sub tlvs of the specific type
-	for t, tlv := range stlvs {
-		switch t {
-		case 1:
-			// Sub Sub TLV Type 1 is SRv6 Structure Sub Sub TLV
-			for _, stlvRaw := range tlv.Tlv {
-				// Instantiating Information Sub TLV
-				structure := &SRv6SIDStructureSubSubTLV{
-					SubSubTLV: SubSubTLV{
-						Type:   SubSubTLVType(1),
-						Length: 6,
-					},
-				}
-				var raw ptypes.DynamicAny
-				if err := ptypes.UnmarshalAny(stlvRaw, &raw); err != nil {
-					return 0, nil, err
-				}
-				structureProto := raw.Message.(*api.SRv6StructureSubSubTLV)
-				structure.LocalBlockLength = uint8(structureProto.LocalBlockLength)
-				structure.LocatorNodeLength = uint8(structureProto.LocalNodeLength)
-				structure.FunctionLength = uint8(structureProto.FunctionLength)
-				structure.ArgumentLength = uint8(structureProto.ArgumentLength)
-				structure.TranspositionLength = uint8(structureProto.TranspositionLength)
-				structure.TranspositionOffset = uint8(structureProto.TranspositionOffset)
+func (t *SRv6ServiceTLV) MarshalJSON() ([]byte, error) {
+	return json.Marshal(struct {
+		Type    TLVType `json:"type"`
+		SubTLVs []PrefixSIDTLVInterface
+	}{
+		t.Type,
+		t.SubTLVs,
+	})
+}
 
-				// SRv6 Structure Sub Sub TLV length consists of header 3 bytes, 6 bytes of value
-				l += (3 + 6)
-				p = append(p, structure)
-			}
-		default:
-			return 0, nil, fmt.Errorf("unknown or not implemented Prefix SID Sub TLV type: %d", t)
-		}
+func (t *SRv6ServiceTLV) String() string {
+	var buf bytes.Buffer
+
+	for _, tlv := range t.SubTLVs {
+		buf.WriteString(fmt.Sprintf("%s ", tlv.String()))
 	}
 
-	return l, p, nil
+	return fmt.Sprintf("{SRv6 Service TLV: %s}", buf.String())
 }
