@@ -19,6 +19,7 @@ package server
 
 import (
 	"bytes"
+	"encoding/binary"
 	"syscall"
 	"testing"
 	"unsafe"
@@ -31,15 +32,14 @@ func Test_buildTcpMD5Sig(t *testing.T) {
 		t.Error("TCPM5Sig struct size is wrong", unsafe.Sizeof(s))
 	}
 
-	buf1 := make([]uint8, 216)
-	p := unsafe.Pointer(&s)
-	for i := uintptr(0); i < 216; i++ {
-		buf1[i] = *(*byte)(unsafe.Pointer(uintptr(p) + i))
+	buf1 := new(bytes.Buffer)
+	if err := binary.Write(buf1, binary.LittleEndian, s); err != nil {
+		t.Error(err)
 	}
 
 	buf2 := []uint8{2, 0, 0, 0, 1, 2, 3, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 104, 101, 108, 108, 111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
 
-	if bytes.Equal(buf1, buf2) {
+	if bytes.Equal(buf1.Bytes(), buf2) {
 		t.Log("OK")
 	} else {
 		t.Error("Something wrong v4")
@@ -49,17 +49,16 @@ func Test_buildTcpMD5Sig(t *testing.T) {
 func Test_buildTcpMD5Sigv6(t *testing.T) {
 	s, _ := buildTcpMD5Sig("fe80::4850:31ff:fe01:fc55", "helloworld")
 
-	buf1 := make([]uint8, 216)
-	p := unsafe.Pointer(&s)
-	for i := uintptr(0); i < 216; i++ {
-		buf1[i] = *(*byte)(unsafe.Pointer(uintptr(p) + i))
+	buf1 := new(bytes.Buffer)
+	if err := binary.Write(buf1, binary.LittleEndian, s); err != nil {
+		t.Error(err)
 	}
 
 	buf2 := []uint8{10, 0, 0, 0, 0, 0, 0, 0, 254, 128, 0, 0, 0, 0, 0, 0, 72, 80, 49, 255, 254, 1, 252, 85, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 104, 101, 108, 108, 111, 119, 111, 114, 108, 100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
 
 	buf2[0] = syscall.AF_INET6
 
-	if bytes.Equal(buf1, buf2) {
+	if bytes.Equal(buf1.Bytes(), buf2) {
 		t.Log("OK")
 	} else {
 		t.Error("Something wrong v6")
