@@ -14,7 +14,6 @@
 # limitations under the License.
 
 
-
 from itertools import combinations
 import sys
 import time
@@ -984,7 +983,7 @@ class GoBGPTestBase(unittest.TestCase):
 
 if __name__ == '__main__':
     output = local("which docker 2>&1 > /dev/null ; echo $?", capture=True)
-    if int(output) is not 0:
+    if int(output) != 0:
         print("docker not found")
         sys.exit(1)
 
