@@ -18,5 +18,5 @@
 package tools
 
 import (
-	_ "github.com/golang/protobuf/protoc-gen-go"
+	_ "google.golang.org/protobuf/cmd/protoc-gen-go"
 )
